/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "point.h"

struct ctx
{
  GtkWidget *draw;
  GdkPixbuf *picture;
  GArray *points;
  int i;
  gboolean move;
};

#define FILENAME "/home/cascardo/fotos/debconf.jpg"
#define WIDTH 800
#define HEIGHT 600

gboolean
queue (gpointer data)
{
  struct ctx *ctx;
  ctx = (struct ctx *) data;
  gtk_widget_queue_draw (GTK_WIDGET (ctx->draw));
  ctx->move = TRUE;
  return FALSE;
}

gboolean
expose (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  GdkPixbuf *screen;
  struct ctx *ctx;
  Point point;
  int w, h;
  ctx = (struct ctx *) data;
  gdk_drawable_get_size (GDK_DRAWABLE (event->window), &w, &h);
  if (ctx->move)
    ctx->i = (ctx->i >= ctx->points->len) ? 0 : ctx->i + 1;
  point = g_array_index (ctx->points, Point, ctx->i);
  screen = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
                           event->area.width, event->area.height);
  gdk_pixbuf_scale (ctx->picture, screen, 0, 0,
                    event->area.width, event->area.height,
		    -point.x + w/2 - event->area.x,
		    -point.y + h/2 - event->area.y,
		    point.rx, point.ry, GDK_INTERP_BILINEAR);
  gdk_draw_pixbuf (widget->window, NULL, screen, 0, 0,
                   event->area.x, event->area.y,
		   event->area.width, event->area.height,
                   GDK_RGB_DITHER_NONE, 0, 0);
  gdk_pixbuf_unref (screen);
  if (point.name)
    {
      if (ctx->move)
        g_timeout_add (3000, queue, ctx);
    }
  else
    {
      if (ctx->move)
        g_timeout_add (10, queue, ctx);
    }
  ctx->move = FALSE;
  return FALSE;
}

int
main (int argc, char **argv)
{
  char *filename;
  GdkColorspace colorspace;
  gboolean has_alpha;
  int bits_per_sample;
  int width, height;
  GtkWidget *window;
  struct ctx ctx;
  gtk_init (&argc, &argv);
  if (argc < 2)
    filename = FILENAME;
  else
    filename = argv[1];
  ctx.points = ReadPoints ("pro-gnu");
  ctx.points = drop_dup_frames (ctx.points, 20);
  rescale_points (ctx.points, get_scales (20));
  ctx.picture = gdk_pixbuf_new_from_file (filename, NULL);
  ctx.i = ctx.points->len;
  colorspace = gdk_pixbuf_get_colorspace (ctx.picture);
  has_alpha = gdk_pixbuf_get_has_alpha (ctx.picture);
  bits_per_sample = gdk_pixbuf_get_bits_per_sample (ctx.picture);
  width = WIDTH;
  height = HEIGHT;
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (G_OBJECT (window), "destroy",
                    G_CALLBACK (gtk_main_quit), NULL);
  ctx.draw = gtk_drawing_area_new ();
  gtk_widget_set_size_request (ctx.draw, width, height);
  gtk_container_add (GTK_CONTAINER (window), ctx.draw);
  gtk_widget_show_all (window);
  g_signal_connect (G_OBJECT (ctx.draw), "expose_event",
                    G_CALLBACK (expose), &ctx);
  g_timeout_add (10, queue, &ctx);
  gtk_main ();
  gdk_pixbuf_unref (ctx.picture);
  return 0;
}
